/// @author: iMujtaba Nazki
/// Date: 30-04-2019
/// Copyright: im8488 Nazki @: 2019

import 'package:flutter/material.dart';

/// A ThreeColumnTextBar is typically used to display a textual summary bar
/// along the top of the device screen. It has 3 columns marked as c1, c2, and
/// c3. Each column in turn has two fixed rows, wherein, the first row is used
/// to display a label which defines the data that is contained within the
/// second row. The labelText size is smaller than the dataTextSize. Examples
/// where it can be used includes, to display income / expense summary in a
/// Personal Finance Management app, whether forecast for 3 days in a weather
/// app, etc. Keep in mind that it only displays the information using 'text',
/// hence, graphics and other widgets are not supported, as of yet.
class ThreeColumnTextBar extends StatelessWidget {
  final double _holderHeight;
  final double _holderBorderRadius;
  final Color _holderColor;
  final String _c1LabelText;
  final String _c1DataText;
  final String _c2LabelText;
  final String _c2DataText;
  final String _c3LabelText;
  final String _c3DataText;
  final Color _labelTextColor;
  final double _labelTextFontSize;
  final FontWeight _labelTextFontWeight;
  final String _labelTextFontFamily;
  final Color _dataTextColor;
  final double _dataTextFontSize;
  final FontWeight _dataTextFontWeight;
  final String _dataTextFontFamily;
  final bool _showHangingDivider;
  final Color _hangingDividerColor;

  /// It allows you to create the most configurable ThreeColumnTextBar with
  /// customizable default grey background. The labelText and dataText are set
  /// to blue and black colors, respectively. Also borderRadius is set to 0 and
  /// hangingDivider is visible. The prefix of c1, c2, and c3 represent Column
  /// #1, Column #2, and Column #3, respectively. Note: The minimum holder
  /// hieght is 60px on any device.
  ThreeColumnTextBar({
    double holderHeight = 60.0,
    double holderBorderRadius = 0.0,
    Color holderColor = Colors.grey,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    Color labelTextColor = Colors.blue,
    double labelTextFontSize = 12.0,
    FontWeight labelTextFontWeight = FontWeight.normal,
    String labelTextFontFamily,
    Color dataTextColor = Colors.black,
    double dataTextFontSize = 14.0,
    FontWeight dataTextFontWeight = FontWeight.normal,
    String dataTextFontFamily,
    bool showHangingDivider = true,
    Color hangingDividerColor = Colors.white,
  })  : _c1LabelText = c1LabelText,
        _c1DataText = c1DataText,
        _c2LabelText = c2LabelText,
        _c2DataText = c2DataText,
        _c3LabelText = c3LabelText,
        _c3DataText = c3DataText,
        _holderHeight = holderHeight >= 60.0 ? holderHeight : 60.0,
        _labelTextColor = labelTextColor,
        _labelTextFontSize = labelTextFontSize,
        _labelTextFontWeight = labelTextFontWeight,
        _labelTextFontFamily = labelTextFontFamily,
        _dataTextColor = dataTextColor,
        _dataTextFontSize = dataTextFontSize,
        _dataTextFontWeight = dataTextFontWeight,
        _dataTextFontFamily = dataTextFontFamily,
        _holderColor = holderColor,
        _holderBorderRadius = holderBorderRadius,
        _showHangingDivider = showHangingDivider,
        _hangingDividerColor = hangingDividerColor;

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: _holder(context: context),
    );
  }

  /// Holder is the primary container that holds all other components within the
  /// ThreeColumnTextBar. It has a minimum height of 60px on any device and its
  /// width is the width of the device's screen.
  Widget _holder({@required BuildContext context}) {
    return Container(
      width: MediaQuery.of(context).size.width,
      height: _holderHeight,
      margin: EdgeInsets.all(5.0),
      decoration: BoxDecoration(
        color: _holderColor,
        borderRadius: BorderRadius.circular(_holderBorderRadius),
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          _textColumn(
            context: context,
            labelText: _c1LabelText,
            dataText: _c1DataText,
          ),
          _showHangingDivider ? _hangingVerticalDivider() : Container(),
          _textColumn(
            context: context,
            labelText: _c2LabelText,
            dataText: _c2DataText,
          ),
          _showHangingDivider ? _hangingVerticalDivider() : Container(),
          _textColumn(
            context: context,
            labelText: _c3LabelText,
            dataText: _c3DataText,
          ),
        ],
      ),
    );
  }

  /// A [_textColumn] represents a single column within a ThreeColumnTextBar,
  /// which is configured to hold the label and the data Text in a top-down
  /// order.
  Widget _textColumn({
    @required BuildContext context,
    String labelText = 'labelText',
    String dataText = 'dataText',
  }) {
    return Center(
      child: Container(
        width: MediaQuery.of(context).size.width / 4.0,
        padding: const EdgeInsets.all(8.0),
        child: FittedBox(
          fit: BoxFit.contain,
          child: Column(
            children: <Widget>[
              Text(
                labelText,
                style: TextStyle(
                  color: _labelTextColor,
                  fontSize: _labelTextFontSize,
                  fontWeight: _labelTextFontWeight,
                  fontFamily: _labelTextFontFamily,
                ),
              ),
              SizedBox(
                height: 3.0,
              ),
              Text(
                dataText,
                style: TextStyle(
                  color: _dataTextColor,
                  fontSize: _dataTextFontSize,
                  fontWeight: _dataTextFontWeight,
                  fontFamily: _dataTextFontFamily,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  /// Customized Divider that separates the columns within a ThreeColumnTextBar.
  /// The height of the divider is adjusted by dividing holderHeight with a
  /// factor of 2.5
  Widget _hangingVerticalDivider() {
    return SizedBox(
      height: _holderHeight / 2.5,
      child: VerticalDivider(
        color: _hangingDividerColor,
      ),
    );
  }

  //----------------------- PRE-CONFIGURED -----------------------------------//

  /// Constructs a ThreeColumnTextBar with transparent background and effects.
  /// The holder backgroundColor is set to transparent, labelTextColor to deep
  /// purple accent, dataTextColor to black, and dividerColor to deep purple.
  ThreeColumnTextBar.transparent({
    double holderHeight = 60.0,
    double holderBorderRadius = 0.0,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    double labelTextFontSize = 10.0,
    FontWeight labelTextFontWeight = FontWeight.normal,
    String labelTextFontFamily,
    double dataTextFontSize = 12.0,
    FontWeight dataTextFontWeight = FontWeight.normal,
    String dataTextFontFamily,
    bool showHangingDivider = true,
  }) : this(
          holderHeight: holderHeight,
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          labelTextFontSize: labelTextFontSize,
          labelTextFontWeight: labelTextFontWeight,
          labelTextFontFamily: labelTextFontFamily,
          dataTextFontSize: dataTextFontSize,
          dataTextFontWeight: dataTextFontWeight,
          dataTextFontFamily: dataTextFontFamily,
          showHangingDivider: showHangingDivider,

          // Pre-configured.
          holderColor: Colors.transparent,
          labelTextColor: Colors.deepPurpleAccent,
          dataTextColor: Colors.black,
          hangingDividerColor: Colors.deepPurple,
        );

  /// Constructs a ThreeColumnTextBar with dark background and effects. The
  /// holder backgroundColor is set to black, labelTextColor to yellow, and
  /// dataTextColor to white.
  ThreeColumnTextBar.dark({
    double holderHeight = 60.0,
    double holderBorderRadius = 0.0,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    double labelTextFontSize = 10.0,
    FontWeight labelTextFontWeight = FontWeight.normal,
    String labelTextFontFamily,
    double dataTextFontSize = 12.0,
    FontWeight dataTextFontWeight = FontWeight.normal,
    String dataTextFontFamily,
    bool showHangingDivider = true,
  }) : this(
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          holderHeight: holderHeight,
          labelTextFontSize: labelTextFontSize,
          labelTextFontWeight: labelTextFontWeight,
          labelTextFontFamily: labelTextFontFamily,
          dataTextFontSize: dataTextFontSize,
          dataTextFontWeight: dataTextFontWeight,
          dataTextFontFamily: dataTextFontFamily,
          holderBorderRadius: holderBorderRadius,
          showHangingDivider: showHangingDivider,

          // Pre-configured.
          holderColor: Colors.black,
          labelTextColor: Colors.yellow,
          dataTextColor: Colors.white,
          hangingDividerColor: Colors.yellow,
        );

  /// Constructs a ThreeColumnTextBar with deepPurple background and effects.
  /// The holder backgroundColor is set to deep purple, labelTextColor to
  /// yellow, and dataTextColor to white.
  ThreeColumnTextBar.deepPurple({
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    double holderHeight = 60.0,
    double labelTextFontSize = 10.0,
    FontWeight labelTextFontWeight = FontWeight.normal,
    String labelTextFontFamily,
    double dataTextFontSize = 12.0,
    FontWeight dataTextFontWeight = FontWeight.normal,
    String dataTextFontFamily,
    double holderBorderRadius = 0.0,
    bool showHangingDivider = true,
  }) : this(
          holderHeight: holderHeight,
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          labelTextFontSize: labelTextFontSize,
          labelTextFontWeight: labelTextFontWeight,
          labelTextFontFamily: labelTextFontFamily,
          dataTextFontSize: dataTextFontSize,
          dataTextFontWeight: dataTextFontWeight,
          dataTextFontFamily: dataTextFontFamily,
          showHangingDivider: showHangingDivider,
          holderBorderRadius: holderBorderRadius,

          // Pre-configured.
          holderColor: Colors.deepPurple,
          labelTextColor: Colors.yellow,
          dataTextColor: Colors.white,
        );
}
